
// Created by fizzim.pl version 4.41 on 2013:03:13 at 15:42:25 (www.fizzim.com)

module vme_strobe_sim_FSM (
  output reg AS,
  output reg DS0,
  output reg DS1,
  input CLK,
  input DTACK,
  input GO,
  input RST 
);
  
  // state bits
  parameter 
  Idle    = 3'b000, 
  RmvAS   = 3'b001, 
  RmvDS1  = 3'b010, 
  Rmv_DS0 = 3'b011, 
  SET_AS  = 3'b100, 
  SET_DS0 = 3'b101, 
  SET_DS1 = 3'b110; 
  
  reg [2:0] state;
  reg [2:0] nextstate;
  
  // comb always block
  always @* begin
    nextstate = 3'bxxx; // default to x because default_state_is_x is set
    case (state)
      Idle   : if (GO)     nextstate = SET_AS;
               else        nextstate = Idle;
      RmvAS  : if (!GO)    nextstate = Idle;
      RmvDS1 :             nextstate = RmvAS;
      Rmv_DS0:             nextstate = RmvDS1;
      SET_AS :             nextstate = SET_DS0;
      SET_DS0:             nextstate = SET_DS1;
      SET_DS1: if (DTACK)  nextstate = Rmv_DS0;
               else        nextstate = SET_DS1;
    endcase
  end
  
  // Assign reg'd outputs to state bits
  
  // sequential always block
  always @(posedge CLK or posedge RST) begin
    if (RST)
      state <= Idle;
    else
      state <= nextstate;
  end
  
  // datapath sequential always block
  always @(posedge CLK or posedge RST) begin
    if (RST) begin
      AS <= 0;
      DS0 <= 0;
      DS1 <= 0;
    end
    else begin
      AS <= 0; // default
      DS0 <= 0; // default
      DS1 <= 0; // default
      case (nextstate)
        RmvDS1 :        AS <= 1;
        Rmv_DS0: begin
                        AS <= 1;
                        DS1 <= 1;
        end
        SET_AS :        AS <= 1;
        SET_DS0: begin
                        AS <= 1;
                        DS0 <= 1;
        end
        SET_DS1: begin
                        AS <= 1;
                        DS0 <= 1;
                        DS1 <= 1;
        end
      endcase
    end
  end
  
  // This code allows you to see state names in simulation
  `ifndef SYNTHESIS
  reg [55:0] statename;
  always @* begin
    case (state)
      Idle   : statename = "Idle";
      RmvAS  : statename = "RmvAS";
      RmvDS1 : statename = "RmvDS1";
      Rmv_DS0: statename = "Rmv_DS0";
      SET_AS : statename = "SET_AS";
      SET_DS0: statename = "SET_DS0";
      SET_DS1: statename = "SET_DS1";
      default: statename = "XXXXXXX";
    endcase
  end
  `endif

endmodule

