library ieee;
use ieee.std_logic_1164.all;
USE IEEE.std_logic_arith.all;
USE IEEE.std_logic_unsigned.all;

ENTITY dmb_vme_sim IS
PORT 	(

-- VME signals

		vme_addr : IN STD_LOGIC_VECTOR (23 downto 1);			-- adr(23 downto 1)
		vme_data_in : IN STD_LOGIC_VECTOR (15 downto 0);		-- data(15 downto 0)
		vme_data_out : OUT STD_LOGIC_VECTOR (15 downto 0);		-- data(15 downto 0)
		vme_am : IN STD_LOGIC_VECTOR (5 downto 0);				-- am(5 downto 0)
		vme_ga : IN STD_LOGIC_VECTOR (4 downto 0);				-- ga*(4 downto 0)
		vme_gap : IN STD_LOGIC;											-- ga*(5)
		vme_ds_b : IN STD_LOGIC_VECTOR(1 DOWNTO 0);				-- ids1*,ids0*
		vme_as_b : IN STD_LOGIC;										-- ias*
		vme_lword_b : IN STD_LOGIC;									-- ilword*
		vme_write_b : IN STD_LOGIC;									-- iwrite*
		vme_iack_b : IN STD_LOGIC;										-- inack*
		vme_sysreset_b : IN STD_LOGIC;								-- isysrst*
		vme_sysfail_b : IN STD_LOGIC;									-- isysfail*
		vme_sysfail_out : OUT STD_LOGIC;								-- NEW
		vme_berr_b : IN STD_LOGIC;										-- iberr*
		vme_berr_out : OUT STD_LOGIC;									-- NEW
		vme_dtack_b : INOUT STD_LOGIC;									-- odtack* INOUT by bgb
		vme_tovme : OUT STD_LOGIC;										-- tovme
		vme_tovme_b : OUT STD_LOGIC;									-- tovme*
		vme_doe : OUT STD_LOGIC;										-- doe
		vme_doe_b : OUT STD_LOGIC;										-- doe*

-- Clock
		
		clk : IN STD_LOGIC;												-- NEW (fastclk -> 40MHz)
		clk_s1 : IN STD_LOGIC;											-- NEW (midclk -> fastclk/4 -> 10MHz)
		clk_s2 : IN STD_LOGIC;											-- NEW (slowclk -> midclk/4 -> 2.5MHz)
		clk_s3 : IN STD_LOGIC;											-- NEW (slowclk2 -> midclk/8 -> 12.5MHz)

-- Reset

		rst : IN STD_LOGIC;												-- iglobalrst

-- To/From BPI_PORT 

	BPI_RST           : out std_logic;                      -- Resets BPI interface state machines
	BPI_CMD_FIFO_DATA : out std_logic_vector(15 downto 0);  -- Data for command FIFO
	BPI_WE            : out std_logic;                      -- Command FIFO write enable  (pulse one clock cycle for one write)
	BPI_RE            : out std_logic;                      -- Read back FIFO read enable  (pulse one clock cycle for one read)
	BPI_DSBL          : out std_logic;                      -- Disable parsing of BPI commands in the command FIFO (while being filled)
	BPI_ENBL          : out std_logic;                      -- Enable  parsing of BPI commands in the command FIFO
	BPI_RBK_FIFO_DATA : in std_logic_vector(15 downto 0); -- Data on output of the Read back FIFO
	BPI_RBK_WRD_CNT   : in std_logic_vector(10 downto 0); -- Word count of the Read back FIFO (number of available reads)
	BPI_STATUS        : in std_logic_vector(15 downto 0); -- FIFO status bits and latest value of the PROM status register. 
	BPI_TIMER         : in std_logic_vector(31 downto 0)  -- General timer

);
	
end dmb_vme_sim;


ARCHITECTURE dmb_vme_sim_architecture OF dmb_vme_sim IS

signal ext_vme_ga  : std_logic_vector(5 downto 0);

signal device : std_logic_vector(9 downto 0);
signal cmd : std_logic_vector(9 downto 0);
signal strobe : std_logic;
-- bgb (not used) signal vme_ready : std_logic;
signal tovme_b, doe_b : std_logic;


signal diagout_command  : std_logic_vector(19 downto 0);
signal led_command  : std_logic_vector(2 downto 0);

signal cmd_adrs  : std_logic_vector(15 downto 0);
signal cmd_vme_data_out, cmd_vme_data_in, cmd_vme_data  : std_logic_vector(15 downto 0);

signal outdata_bpi_port  : std_logic_vector(15 downto 0);

COMPONENT BPI_PORT is
  
  port (

    CLK : in std_logic;                          -- 40MHz clock
    RST : in std_logic;                          -- system reset
    -- VME selection/control
    DEVICE : in std_logic;                       -- 1 bit indicating this device has been selected
    STROBE : in std_logic;                       -- Data strobe synchronized to rising or falling edge of clock and asynchronously cleared
    COMMAND : in std_logic_vector(9 downto 0);   -- command portion of VME address
    WRITE_B : in std_logic;                      -- read/write_bar
    INDATA : in std_logic_vector(15 downto 0);   -- data from VME writes to be provided to BPI interface
    OUTDATA : out std_logic_vector(15 downto 0); -- data from BPI interface to VME buss for reads
    DTACK_B : out std_logic;                     -- DTACK bar
	-- BPI PORT signals
	BPI_RST           : out std_logic;                      -- Resets BPI interface state machines
	BPI_CMD_FIFO_DATA : out std_logic_vector(15 downto 0);  -- Data for command FIFO
	BPI_WE            : out std_logic;                      -- Command FIFO write enable  (pulse one clock cycle for one write)
	BPI_RE            : out std_logic;                      -- Read back FIFO read enable  (pulse one clock cycle for one read)
	BPI_DSBL          : out std_logic;                      -- Disable parsing of BPI commands in the command FIFO (while being filled)
	BPI_ENBL          : out std_logic;                      -- Enable  parsing of BPI commands in the command FIFO
	BPI_RBK_FIFO_DATA : in std_logic_vector(15 downto 0); -- Data on output of the Read back FIFO
	BPI_RBK_WRD_CNT   : in std_logic_vector(10 downto 0); -- Word count of the Read back FIFO (number of available reads)
	BPI_STATUS        : in std_logic_vector(15 downto 0); -- FIFO status bits and latest value of the PROM status register. 
	BPI_TIMER         : in std_logic_vector(31 downto 0)  -- General timer
);
end COMPONENT;

COMPONENT COMMAND_MODULE is
  
  port (

-- Chip Scope Pro Logic Analyzer control -- bgb

--    CSP_VME_LA_CTRL: inout std_logic_vector(35 downto 0); -- bgb
--    VME_LA_TRG : inout std_logic; -- bgb
	 
    FASTCLK: in std_logic;
    SLOWCLK: in std_logic;

    GA : in std_logic_vector(5 downto 0);
    ADR : in std_logic_vector(23 downto 1);
    AM : in std_logic_vector(5 downto 0);

    AS : in std_logic;
    DS0 : in std_logic;
    DS1 : in std_logic;
    LWORD : in std_logic;
    WRITER : in std_logic; 
    IACK : in std_logic;
    BERR : in std_logic;
    SYSFAIL : in std_logic;
    DTACK_B : in std_logic; -- bgb

    DEVICE   : inout std_logic_vector(9 downto 0);
    STROBE   : inout std_logic;
    COMMAND  : inout std_logic_vector(9 downto 0);
    ADRS     : out std_logic_vector(17 downto 2);  

-- bgb   INDATA   : out std_logic_vector(15 downto 0);
-- bgb   OUTDATA : in std_logic_vector(15 downto 0);

    VME_DATA : in std_logic_vector(15 downto 0);
    TOVME_B    : out std_logic;
    DOE_B    : out std_logic;
-- bgb (not used)   VMEREADY : out std_logic;


    DIAGOUT  : out std_logic_vector(19 downto 0);
    LED      : out std_logic_vector(2 downto 0)

    );
    
end COMPONENT;


begin

vme_doe_b <= doe_b;

vme_doe <= not doe_b;

vme_tovme_b <= tovme_b;

vme_tovme <= not tovme_b;

vme_sysfail_out <= '0';

vme_berr_out <= '0';

cmd_vme_data_out <= "0000000000000000";

ext_vme_ga <= vme_gap & vme_ga;

PMAP_COMMAND_MODULE : COMMAND_MODULE
  
PORT MAP (

-- Chip Scope Pro Logic Analyzer control -- bgb
--
--    CSP_VME_LA_CTRL => 35'h000000000, -- bgb -- inout
--    VME_LA_TRG => VME_LA_TRG, -- bgb

    FASTCLK => clk,
    SLOWCLK => clk_s2,

	 GA => ext_vme_ga, -- gap = ga(5)
    ADR => vme_addr,
    AM => vme_am,

    AS => vme_as_b,
    DS0 => vme_ds_b(0),
    DS1 => vme_ds_b(1),
    LWORD => vme_lword_b,
    WRITER => vme_write_b,
    IACK => vme_iack_b,
    BERR => vme_berr_b,
    SYSFAIL => vme_sysfail_b,
    DTACK_B => vme_dtack_b,

-- bgb   INDATA => cmd_vme_data_in,
-- bgb   OUTDATA => cmd_vme_data_out,
--	bgb DATA => cmd_vme_data, -- not used (tri-state at the top level)
	 VME_DATA => vme_data_in, -- bgb VME data buss (in and out) for logic anlyzer

    TOVME_B => tovme_b,
    DOE_B => doe_b,
-- bgb (not used)    VMEREADY => vme_ready,

 	 DEVICE => device,
    STROBE => strobe,
    COMMAND => cmd,
    ADRS => cmd_adrs,

    DIAGOUT => diagout_command,
    LED => led_command
    );

vme_data_sel : process (device,outdata_bpi_port)
begin		
	case device is
		when "0000010000" =>	vme_data_out <= outdata_bpi_port;
		when others =>	vme_data_out <= x"0000";
	end case;
  
end process;
	
-- Device 4 => BPI_PORT
BPI_PORT_i : BPI_PORT

PORT MAP (

    CLK => clk,                             -- 40MHz clock
    RST => rst,                             -- system reset
	 -- VME selection/control
    DEVICE => device(4),                    -- 1 bit indicating this device has been selected
    STROBE => strobe,                       -- Data strobe synchronized to rising or falling edge of clock and asynchronously cleared
    COMMAND => cmd,                         -- command portionn of VME address
    WRITE_B => vme_write_b,                 -- read/write_bar
    INDATA => vme_data_in,                  -- data from VME writes to be provided to BPI interface
    OUTDATA => outdata_bpi_port,            -- data from BPI interface to VME buss for reads
    DTACK_B => vme_dtack_b,                 -- DTACK bar
    -- BPI controls
	BPI_RST           => BPI_RST,            -- Resets BPI interface state machines
	BPI_CMD_FIFO_DATA => BPI_CMD_FIFO_DATA,  -- Data for command FIFO
	BPI_WE            => BPI_WE,             -- Command FIFO write enable  (pulse one clock cycle for one write)
	BPI_RE            => BPI_RE,             -- Read back FIFO read enable  (pulse one clock cycle for one read)
	BPI_DSBL          => BPI_DSBL,           -- Disable parsing of BPI commands in the command FIFO (while being filled)
	BPI_ENBL          => BPI_ENBL,           -- Enable  parsing of BPI commands in the command FIFO
	BPI_RBK_FIFO_DATA => BPI_RBK_FIFO_DATA,  -- Data on output of the Read back FIFO
	BPI_RBK_WRD_CNT   => BPI_RBK_WRD_CNT,    -- Word count of the Read back FIFO (number of available reads)
	BPI_STATUS        => BPI_STATUS,         -- FIFO status bits and latest value of the PROM status register. 
	BPI_TIMER         => BPI_TIMER           -- General timer
);



end dmb_vme_sim_architecture;

